"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unpackAndVerifySignatureDecorator = unpackAndVerifySignatureDecorator;
exports.signData = signData;
const core_1 = require("@credo-ts/core");
const SignatureDecorator_1 = require("./SignatureDecorator");
/**
 * Unpack and verify signed data before casting it to the supplied type.
 *
 * @param decorator Signature decorator to unpack and verify
 * @param wallet wallet instance
 *
 * @return Resulting data
 */
async function unpackAndVerifySignatureDecorator(decorator, wallet) {
    const signerVerkey = decorator.signer;
    const key = core_1.Key.fromPublicKeyBase58(signerVerkey, core_1.KeyType.Ed25519);
    // first 8 bytes are for 64 bit integer from unix epoch
    const signedData = core_1.TypedArrayEncoder.fromBase64(decorator.signatureData);
    const signature = core_1.TypedArrayEncoder.fromBase64(decorator.signature);
    // const isValid = await wallet.verify(signerVerkey, signedData, signature)
    const isValid = await wallet.verify({ signature, data: signedData, key });
    if (!isValid) {
        throw new core_1.CredoError('Signature is not valid');
    }
    // TODO: return Connection instance instead of raw json
    return core_1.JsonEncoder.fromBuffer(signedData.slice(8));
}
/**
 * Sign data supplied and return a signature decorator.
 *
 * @param data the data to sign
 * @param wallet the wallet containing a key to use for signing
 * @param signerKey signers verkey
 *
 * @returns Resulting signature decorator.
 */
async function signData(data, wallet, signerKey) {
    const dataBuffer = core_1.Buffer.concat([core_1.utils.timestamp(), core_1.JsonEncoder.toBuffer(data)]);
    const key = core_1.Key.fromPublicKeyBase58(signerKey, core_1.KeyType.Ed25519);
    const signatureBuffer = await wallet.sign({ key, data: dataBuffer });
    const signatureDecorator = new SignatureDecorator_1.SignatureDecorator({
        signatureType: 'https://didcomm.org/signature/1.0/ed25519Sha512_single',
        signature: core_1.TypedArrayEncoder.toBase64URL(signatureBuffer),
        signatureData: core_1.TypedArrayEncoder.toBase64URL(dataBuffer),
        signer: signerKey,
    });
    return signatureDecorator;
}
//# sourceMappingURL=SignatureDecoratorUtils.js.map